
#ifdef __cplusplus
extern "C" {
#endif
	#include <stdint.h>
	#include <stdbool.h> 


	/*=========================================================================================
	|								Enums used for sdk C wrapper							   |
	===========================================================================================*/
	typedef enum _SENSORS {
		PM = 0, ADXL, ADPD, ADPD4000, PPG, SYNCPPG, ECG, EDA, AD5940, TEMPERATURE
	}SENSORS;

	typedef enum _ADPD4XXX_SOURCE {
		STREAM1 = 1, STREAM2, STREAM3, STREAM4, STREAM5, STREAM6, STREAM7, STREAM8, STREAM9, STREAM10, STREAM11, STREAM12
	}ADPD4XXX_SOURCE;

	typedef enum _SENSOR_ADPD_DEVICE_ID {
		ADPD_DEVICE_107 = 2,
		ADPD_DEVICE_185,
		ADPD_DEVICE_ECG_PPG_185,
		ADPD_DEVICE_ECG_185,
		ADPD_DEVICE_105,
		ADPD_DEVICE_188,
		ADPD_DEVICE_108,
		ADPD_DEVICE_188F,
		ADPD_DEVICE_ECG_PPG_188,
		ADPD_DEVICE_ECG_188,
		ADPDCL_DEVICE_4000 = 40,
	}SENSOR_ADPD_DEVICE_ID;

	typedef enum _ADPD_SLOT_MODE_ENUM
	{
		SLOTMODE_DISABLED = 0,
		SLOTMODE_4CH_16b = 1,
		SLOTMODE_4CH_32b = 4,
		SLOTMODE_SUM16b = 17,
		SLOTMODE_SUM32b = 20,
	}ADPD_SLOT_MODE_ENUM;

	typedef enum _SDK_STATUS {
		SDK_OK = 0,
		SDK_ERR = 1,
		SDK_ERR_INVALID,
		SDK_DEVICE_ERR,
		SDK_PACKET_TIMEOUT,
	}SDK_STATUS;


	typedef enum _SENSOR_PPG_LCFG_ID {
		PPG_LCFG_ID_ADPD107 = 107,
		PPG_LCFG_ID_ADPD185 = 185,
		PPG_LCFG_ID_ADPD108 = 108,
		PPG_LCFG_ID_ADPD188 = 188,
		PPG_LCFG_ID_ADPD4000 = 40,
	}SENSOR_PPG_LCFG_ID;

	/*=========================================================================================
	|								Structures used for sdk C wrapper						   |
	===========================================================================================*/
//#pragma pack(1)
	typedef struct _registers
	{
		int address[53];
		int value[53];
		short length;
	}registers;

	typedef struct _adxl_stream_t
	{
		uint16_t sequence_number;
		double timestamp;
		int16_t x;
		int16_t y;
		int16_t z;
	}adxl_stream_t;

	typedef struct _common_app_version_t {
		uint16_t major;
		uint16_t minor;
		uint16_t patch;
		char *verstr;
		char *vendstr;
	}common_app_version;

		typedef struct _app_version_t {
    		char *verstr;
    	}app_version;

	typedef struct _sys_date_time {
		uint16_t year;
		uint8_t month;
		uint8_t day;
		uint8_t hour;
		uint8_t minute;
		uint8_t second;
		uint32_t TZ_sec;
	}sys_date_time;

	typedef struct _pm_sys_info {
		uint16_t version;
		char *mac_addr;
		uint32_t device_id;
		uint32_t model_number;
		uint16_t hw_id;
		uint16_t bom_id;
		uint8_t batch_id;
		uint32_t date;
	}pm_sys_info;

	typedef struct _pm_sys_bat_info {
		double timestamp;
		uint8_t bat_chrg_stat;
		uint8_t bat_lvl;
		uint16_t bat_mv;
		uint16_t bat_temp;
	}pm_sys_bat_info;

	typedef struct _eeprom_info {
		uint32_t manufacture_id;
		uint16_t hw_id;
		uint16_t bom_id;
		uint8_t batch_id;
		uint32_t date;
		uint32_t additional_detail;

	}eeprom_info;

	typedef struct _get_slot_info {
		uint8_t  slot_num;
		uint8_t  slot_enable;
		uint16_t slot_format;
		uint8_t  channel_num;
	}get_slot_info;

	typedef struct _adpd4000_stream_cb_data {
		uint16_t seq_num;
		ADPD4XXX_SOURCE src;
		uint8_t  dark;
		uint8_t  signal;
		uint8_t  impulse;
		uint8_t  lit;
		uint8_t  channel_num;
		double  timestamp;
		int adpddata_d[24];
		short adpd_d_length;
		int adpddata_s[24];
		short adpd_s_length;
		short adpd_l_length;
		int adpddata_l[24];
	}adpd4000_stream_cb_data;

	typedef struct _adpd4000_interrupt_stream_cb_data_t {
		uint16_t seq_num;
		double  timestamp;
		uint16_t  data_int;
		uint16_t  level0_int;
		uint16_t  level1_int;
		uint16_t  tia_ch1_int;
		uint16_t  tia_ch2_int;
	}adpd4000_interrupt_stream_cb_data_t;




	typedef struct _ppg_stream_cb_data {
		uint16_t seqnum;
		double timestamp;    //!< System timestamp in milliseconds
		uint16_t adpdlibstate; //!< Internal ADPDlib state
		float hr;           //!< PPG-derived heart rate
		float confidence;   //!< Heart rate confidence level
		uint16_t hr_type;      //!< Internal HR type
		uint16_t rr_interval;  //!< R-R peak interval
	}ppg_stream_cb_data;

	typedef struct _syncppg_stream_cb_data {
		uint16_t seqnum;
		double timestamp;
		uint32_t ppg;
		int16_t x;
		int16_t y;
		int16_t z;
	}syncppg_stream_cb_data;


	typedef struct _adpd_stream_cb_data_t_sum {
		uint16_t seqnum;
		double    timestamp;  //!< System timestamp in milliseconds
		uint32_t  adpd_data;  //!< ADPD data value
	}adpd_stream_cb_data_t_sum;

	typedef struct _adpd_stream_cb_data_t_4ch {
		uint16_t seqnum;
		double    timestamp;  //!< System timestamp in milliseconds
		uint32_t  ch_data[4];  //!< ADPD data value
	}adpd_stream_cb_data_t_4ch;

	typedef struct _eda_stream_cb_data {
		uint16_t seqnum;
		double timestamp;  //!< System timestamp in milliseconds
		double admittance_real;
		double admittance_img;
		double impedance_real;
		double impedance_img;
		double admittance_magnitude;
		double admittance_phase;
		double impedance_magnitude;
		double impedance_phase;
	}eda_stream_cb_data;

	typedef struct _ecg_stream_cb_data {
		uint16_t seqnum;
		double timestamp;  //!< System timestamp in milliseconds
		uint8_t datatype;   //!< ECG device type 0 - sport, 1 - monitor
		bool leadsoff;   //!< ECG lead detection
		uint8_t HR;         //!< ECG algorithm heart rate
		uint16_t ecg_data;   //!< ECG data value
	}ecg_stream_cb_data;

	typedef struct _temperature_stream_cb_data {
		double timestamp;    //!< System timestamp in milliseconds
		float temp_skin;    //!< Skin temperature in degrees Celcius
		float temp_ambient; //!< Ambient temperature in degrees Celcius
	}temperature_stream_cb_data;


//#pragma pack()
	/*=========================================================================================
	|									Callbacks											   |
	===========================================================================================*/
	typedef void( *Tx_Callback)(char *pBytes, int nLength);
	typedef void( *adxl_stream_Cb)(adxl_stream_t *pData);
	typedef void( *adpd_stream_sum)(adpd_stream_cb_data_t_sum *pData);
	typedef void( *adpd_stream_4ch)(adpd_stream_cb_data_t_4ch *pData);
	typedef void( *battery_stream_cb)(pm_sys_bat_info *pBatInfo);
	typedef void( *adpd4xxx_stream_cb)(adpd4000_stream_cb_data *pData);
	typedef void( *ppg_stream_cb)(ppg_stream_cb_data *pData);
	typedef void( *syncppg_stream_cb)(syncppg_stream_cb_data *pData);
	typedef void( *ecg_stream_cb)(ecg_stream_cb_data *pData);
	typedef void( *eda_stream_cb)(eda_stream_cb_data *pData);
	typedef void( *adpd4xxx_interrupt_stream_cb)(adpd4000_interrupt_stream_cb_data_t *pData);
	typedef void(*temperature_stream_cb)(temperature_stream_cb_data *pData);

	typedef void( *tx_length)(void);


    typedef void (*add)(int, int);

    typedef void (*sdkAlertCB)(int);

	/*=========================================================================================
	|									SDK TX & RX											   |
	===========================================================================================*/

	int getSDKversion();
	void getLength(tx_length tx_cb);

	int addcb(add cb);

	 void  initializesdk(Tx_Callback tx_cb, sdkAlertCB sdk_cb);
	 void  dispatch_SDK(unsigned char *pBytes, int nLength);


	/*=========================================================================================
	|									PM													   |
	===========================================================================================*/
	 void  pm_get_system_info(pm_sys_info *pSysInfo);
	 void  pm_get_mcu_version(uint8_t *pMCU_Type);
	 SDK_STATUS  pm_set_date_time(sys_date_time *pTimeInfo);
	 void  pm_get_date_time(sys_date_time *pTimeInfo);
	 void  pm_get_battery_info(pm_sys_bat_info *pSysBatInfo);
	 SDK_STATUS  pm_set_battery_threshold(uint8_t nLowLvl, uint8_t nCriticalLvl);
	 SDK_STATUS  pm_set_usb_power(bool enable, bool action);
	 SDK_STATUS  pm_set_power_state(uint8_t nPowerState);
	 SDK_STATUS  pm_ping();
	 SDK_STATUS  pm_enable_or_disable_battery_charger(bool enable);
	 SDK_STATUS  pm_enable_or_disable_thermistor_state_change(bool enable);
	 uint8_t  pm_powerboost_m3(uint8_t nEnableState, uint8_t nOVPValue);
	 uint8_t  pm_powerboost_m4(uint8_t nEnableState);
	 uint8_t  pm_boost_m4(uint8_t nEnableState, uint8_t  nBoostVoltage);
	 uint8_t  pm_read_EEPROM(eeprom_info *pEEPROM_Info);
	 void  pm_subscribe_battery(battery_stream_cb battery_cb);
	 void  pm_unsubscribe_battery();


	/*=========================================================================================
	|									ADXL												   |
	===========================================================================================*/

	 void  subscribe_adxl(adxl_stream_Cb adxl_cb);


	/*=========================================================================================
	|									ADPD												   |
	===========================================================================================*/


	 SDK_STATUS  adpd_loadConfig(SENSOR_ADPD_DEVICE_ID device_id);
	 SDK_STATUS  adpd_clockCalibration(void);
	 void  adpd_darkoffset(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3, uint32_t *pValue4);
	 void  adpd_floatConfig(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3);
	 void  adpd_getctrvalue(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3);
	 void  adpd_set_slot(ADPD_SLOT_MODE_ENUM slot_a, ADPD_SLOT_MODE_ENUM slot_b);
	 void  subscribe_adpd(adpd_stream_sum adpd_sum_cb, adpd_stream_4ch adpd_4ch_cb);


	/*=========================================================================================
	|									ADPD4XXX											   |
	===========================================================================================*/

	 SDK_STATUS  adpd4xxx_load_cfg(void);
	 SDK_STATUS  adpd4xxx_set_slot(uint8_t  nSlotNum, uint8_t  nSlotEnableState, uint16_t  nSlotFormat, uint8_t  nChannelNum);
	 SDK_STATUS  adpd4xxx_get_slot(uint8_t nSlotNum, get_slot_info *pSlotInfo);
	 SDK_STATUS  adpd4xxx_set_slot_active(uint8_t nSlotNum, uint8_t nSlotActvState);
	 SDK_STATUS  adpd4xxx_get_slot_active(uint8_t nSlotNum, uint8_t *pSlotNum, uint8_t *pSlotActvState);
	 SDK_STATUS  adpd4xxx_calibrate_clock(uint8_t nClockCalibID);
	 void  subscribe_adpd4xxx(ADPD4XXX_SOURCE src, adpd4xxx_stream_cb adpd4xxx_cb);
	 void  unsubscribe_adpd4xxx(ADPD4XXX_SOURCE src);
	 void  subscribe_adpd4xxx_interrupt(adpd4xxx_interrupt_stream_cb adpd4xxx_cb);
	 void  unsubscribe_adpd4xxx_interrupt();

	/*=========================================================================================
	|									PPG													   |
	===========================================================================================*/
	 SDK_STATUS ppg_load_lcfg(SENSOR_PPG_LCFG_ID lcfg_id);
	 void subscribe_ppg(ppg_stream_cb ppg_cb);
	 void subscribe_syncppg(syncppg_stream_cb syncppg_cb);
	void getPPGAlgo(app_version *pVersion);


	/*=========================================================================================
	|									EDA													   |
	===========================================================================================*/
	 void subscribe_ecg(ecg_stream_cb ecg_cb);


	/*=========================================================================================
	|									ECG													   |
	===========================================================================================*/
	void subscribe_eda(eda_stream_cb eda_cb);


	/*=========================================================================================
	|									TEMPERATURE													   |
	===========================================================================================*/
	void subscribe_temperature(temperature_stream_cb temp_cb);

	/*=========================================================================================
	|							Common APPlication(ADPD/ADXL/ADPD4000/PPG)					   |
	===========================================================================================*/

	void get_version(SENSORS sensor, common_app_version *pVersion);							// ( ADPD,ADXL,ADPD4000,PPG,PM ,ECG, EDA	)
	void read_registers(SENSORS sensor, unsigned short *pAddress, int nLength, registers *pReg);	// (ADPD,ADXL,ADPD4000)
	void get_dcfg(SENSORS sensor, registers *pReg);												// (ADPD,ADXL,ADPD4000)
	void write_register(SENSORS sensor, registers *pReg);										// (ADPD,ADXL,ADPD4000)
	void start_stream(SENSORS sensor);															// (ADPD,ADXL,ADPD4000,PPG,ECG, EDA,AD5940)
	void stop_stream(SENSORS sensor);															// (ADPD,ADXL,ADPD4000,PPG,ECG, EDA,AD5940)
	void unsubscribe_stream(SENSORS sensor);													// (ADPD,ADXL,PPG,ECG, EDA)
	void setTimeout(SENSORS sensor, uint16_t time);												// ( ADPD,ADXL,ADPD4000,PPG,PM ,ECG, EDA)
	uint16_t getTimeout(SENSORS sensor);														// ( ADPD,ADXL,ADPD4000,PPG,Syncppg,PM, ECG, EDA)
	void stream_status(SENSORS sensor, uint8_t *nSubreq, uint8_t *nStartreq);					// ( ADPD,ADXL,ADPD4000,PPG,SyncPPG , ECG, EDA)
	void get_algo_vendor_version(SENSORS sensor, common_app_version *pVersion);					// (PPG,ECG)
	void  get_lcfg(SENSORS sensor, registers *pReg);												// (PPG,ECG)
	void write_lcfgreg(SENSORS sensor, registers *pReg);										// (PPG,ECG, EDA)
	void read_lfcgreg(SENSORS sensor, unsigned short *pAddress, int nLength, registers *pReg);  // (PPG,ECG, EDA)


	/*=========================================================================================
	|									LEVEL 2 API											   |
	===========================================================================================*/

	void startADXL(adxl_stream_Cb adxl_cb);
	void stopADXL();

	void startADPD(adpd_stream_sum adpd_sum_cb, adpd_stream_4ch adpd_4ch_cb);
	void stopADPD();

	void startPPG(ppg_stream_cb ppg_cb, syncppg_stream_cb syncppg_cb);
	void stopPPG();

	void startADPD4xxx(adpd4xxx_stream_cb adpd4xxx_cb);
	void stopADPD4xxx();

	void startECG(ecg_stream_cb ecg_cb);
	void stopECG();

	void startEDA(eda_stream_cb eda_cb);
	void stopEDA();

	void  startTEMPARATURE(temperature_stream_cb temp_cb);
	void  stopTEMPARATURE();

#ifdef __cplusplus
}
#endif
