/******************************************************************************
* Copyright (c) 2019 Analog Devices, Inc.  All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
* - Redistributions of source code must retain the above copyright notice, this
*   list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Modified versions of the software must be conspicuously marked as such.
* - This software is licensed solely and exclusively for use with
*   processors/products manufactured by or for Analog Devices, Inc.
* - This software may not be combined or merged with other code in any manner
*   that would cause the software to become subject to terms and conditions
*   which differ from those listed here.
* - Neither the name of Analog Devices, Inc. nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
* - The use of this software may or may not infringe the patent rights of one
*   or more patent holders.  This license does not release you from the
*   requirement that you obtain separate licenses from these patent holders to
*   use this software.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* NONINFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
* CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
#pragma once

#include <iostream>
#include <functional>
#include "callback.hpp"
#include "ad5940_application.hpp"
#include "ecg_application.hpp"
#include "adxl_application.hpp"
#include "temperature_application.hpp"
#include "eda_application.hpp"
#include "adpd_application.hpp"
#include "ppg_application.hpp"
#include "syncppg_application.hpp"
#include "pm_application.hpp"
#include "fs_application.hpp"
#include "m2m2/cpp/m2m2_core.hpp"
#include "adpd4000_application.hpp"


/*!
   \brief The top-level watch class that encapsulates all of the SDK applications and operations.
*/
class watch {
private:
  static const std::string git_hash;
  static const std::string git_describe;
  watch_phy_callback &pkt_phy_writer;

public:

enum M2M2_LOW_TOUCH_STATUS_ENUM_t {
    PM_SYS_STATUS_OK = M2M2_PM_SYS_STATUS_ENUM_t::M2M2_PM_SYS_STATUS_OK,
    PM_SYS_STATUS_ERROR = M2M2_PM_SYS_STATUS_ENUM_t::M2M2_PM_SYS_STATUS_ERR_ARGS,
    FILE_SYS_CONFIG_FILE_FOUND = M2M2_FILE_SYS_STATUS_ENUM_t::M2M2_FILE_SYS_CONFIG_FILE_FOUND,
    FILE_SYS_CONFIG_FILE_NOT_FOUND = M2M2_FILE_SYS_STATUS_ENUM_t::M2M2_FILE_SYS_CONFIG_FILE_NOT_FOUND,
    FILE_SYS_STREAM_NOT_STARTED =M2M2_APP_COMMON_STATUS_ENUM_t::M2M2_APP_COMMON_STATUS_STREAM_NOT_STARTED,
    FILE_SYS_STREAM_IN_PROGRESS =M2M2_APP_COMMON_STATUS_ENUM_t::M2M2_APP_COMMON_STATUS_STREAM_IN_PROGRESS,
    FILE_SYS_ERR_CRITICAL_BATTERY_LEVEL = M2M2_FILE_SYS_STATUS_ENUM_t::M2M2_FILE_SYS_STATUS_ERROR,
    FILE_SYS_ERR_MEMORY_FULL=M2M2_FILE_SYS_STATUS_ENUM_t::M2M2_FILE_SYS_ERR_MEMORY_FULL,
    FILE_SYS_ERR_MAX_FILE_COUNT = M2M2_FILE_SYS_STATUS_ENUM_t::M2M2_FILE_SYS_ERR_MAX_FILE_COUNT,
    PM_SYS_STATUS_ENABLE_USER_CONFIG_LOG_FAILED = M2M2_PM_SYS_STATUS_ENUM_t::M2M2_PM_SYS_STATUS_ENABLE_USER_CONFIG_LOG_FAILED,
    PM_SYS_STATUS_USER_CONFIG_LOG_ENABLED = M2M2_PM_SYS_STATUS_ENUM_t::M2M2_PM_SYS_STATUS_USER_CONFIG_LOG_ENABLED,
} M2M2_LOW_TOUCH_STATUS_ENUM_t;
  static const std::string SDK_version;
  enum platforms{python, windows, android, ios} platform;
  watch(watch_phy_callback &cb);
  void self_test(void);
  void set_platform(platforms platForm);
  void dispatch(std::vector<uint8_t> bytes);
  void write_bytes(std::vector<uint8_t> &bytes);
  std::string get_version(void);
  M2M2_ADDR_ENUM_t get_address(void);
  adxl_application adxl_app;
  eda_application eda_app;
  ecg_application ecg_app;
  ad5940_application ad5940_app;
  temperature_application temperature_app;
  adpd_application adpd_app;
  ppg_application ppg_app;
  syncppg_application syncppg_app;
  adpd4000_application adpd4000_app;
  pm_application pm_app;
  fs_application fs_app;

  ret::err_status start_syncppg(ppg_stream_callback &ppg_stream_cb, syncppg_stream_callback &sync_stream_cb);  // Load ADPD & Load PPG for sensor 108
  ret::err_status stop_syncppg(void);

  ret::err_status start_adpd(adpd_stream_callback &stream_cb); // Load ADPD for sensor 108 and set Slot A = 4CH16 , Slot B = SUM32 
  ret::err_status stop_adpd(void);

  ret::err_status start_adxl(adxl_stream_callback &stream_cb);
  ret::err_status stop_adxl(void);

  ret::err_status start_battery(battery_stream_callback &stream_cb);
  ret::err_status stop_battery(void);

  ret::err_status start_adpd4000(adpd4000_stream_callback &stream_cb);
  ret::err_status stop_adpd4000(void);

  ret::err_status start_ecg(ecg_stream_callback &stream_cb);
  ret::err_status stop_ecg(void);

  ret::err_status start_temperature(temperature_stream_callback &stream_cb);

  ret::err_status stop_temperature(void);
  ret::err_status start_eda(eda_stream_callback &stream_cb);
  ret::err_status stop_eda(void);

  m2m2_application::APP_COMMON_STATUS_ENUM_t start_log_adxl(void);
  m2m2_application::APP_COMMON_STATUS_ENUM_t stop_log_adxl(void);
    
  m2m2_application::APP_COMMON_STATUS_ENUM_t start_log_adpd(void);
  m2m2_application::APP_COMMON_STATUS_ENUM_t stop_log_adpd(void);
    
  m2m2_application::APP_COMMON_STATUS_ENUM_t start_log_battery(void);
  m2m2_application::APP_COMMON_STATUS_ENUM_t stop_log_battery(void);

  void system_alert_call(uint8_t alert_message);
};
