#include "CPPtoC_Wrapper.h"
#include "watch.hpp"
#include <string>

static Tx_Callback TX_CB = NULL;
static adxl_stream_Cb ADXL_cb = NULL;
static adpd_stream_sum ADPD_SUM_cb = NULL;
static adpd_stream_4ch ADPD_4CH_cb = NULL;
static battery_stream_cb BATTERY_cb = NULL;
static adpd4xxx_stream_cb ADPD4XXX_cb = NULL;
static ppg_stream_cb PPG_cb = NULL;
static syncppg_stream_cb SYNCPPG_cb = NULL;
static ecg_stream_cb ECG_cb = NULL;
static eda_stream_cb EDA_cb = NULL;
static temperature_stream_cb TEMP_cb = NULL;
static adpd4xxx_interrupt_stream_cb ADPD4XXX_INTERRUPT_cb = NULL;

static sdkAlertCB SDKALERT_CB = NULL;

//#define TXRAWPACKET
//#define RXRAWPACKET

class TXCallback : public watch_phy_callback {
public:
	void call(std::vector<uint8_t> pkt) {
#ifdef TXRAWPACKET
		std::cout << "Tx:" << std::endl;
		for (uint32_t i = 0; i < pkt.size(); i++) {
			std::cout << " " << std::hex << std::setfill('0') << std::setw(2) << (int)pkt[i];
		}
		std::cout << std::endl;
#endif
		
		char *pTxByte = new char[pkt.size()];

		for (int i = 0; i < pkt.size(); i++)
		{
			pTxByte[i] = (char)pkt[i];
		}

		if (TX_CB != NULL)
			TX_CB(pTxByte, pkt.size());

		delete pTxByte;		
	};

	void sys_alert_call(uint8_t sys_alert) {
	    if (SDKALERT_CB != NULL)
	        SDKALERT_CB(sys_alert);
		//std::cout << "sys_alert --: \n" << (int)sys_alert << std::endl;
	};
};

class AdxlStreamCallback : public adxl_stream_callback {
public:

	void call(std::vector<adxl_stream_cb_data_t> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			adxl_stream_cb_data_t *pAdxlItem = &pkt[i];
			adxl_stream_t adxl_data = { 0 };
			adxl_data.sequence_number = sequence_num;
			adxl_data.timestamp = pAdxlItem->timestamp;
			adxl_data.x = pAdxlItem->x;
			adxl_data.y = pAdxlItem->y;
			adxl_data.z = pAdxlItem->z;
			if (ADXL_cb != NULL)
				ADXL_cb(&adxl_data);
			else
				std::cout << "ADXL Callback Not Initialized" << std::endl;
			
		}
	};
};


class AdpdStreamCallback : public adpd_stream_callback {
public:
	
	void call(std::vector<adpd_stream_cb_data_t_sum32> pkt, uint16_t sequence_num) {

		for (uint32_t i = 0; i < pkt.size(); i++) {
			adpd_stream_cb_data_t_sum32 *pAdpdItem = &pkt[i];
			adpd_stream_cb_data_t_sum pData;
			pData.seqnum = sequence_num;
			pData.adpd_data = pAdpdItem->sum32_adpd_data;
			pData.timestamp = pAdpdItem->sum_timestamp;
			if(ADPD_SUM_cb)
				ADPD_SUM_cb(&pData);
			else
				std::cout << "ADPD SUM Callback Not Initialized" << std::endl;

		}

	};

	void call(std::vector<adpd_stream_cb_data_t_sum16> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			adpd_stream_cb_data_t_sum16 *pAdpdItem = &pkt[i];
			adpd_stream_cb_data_t_sum pData = { 0 };

			pData.seqnum = sequence_num;
			pData.adpd_data = pAdpdItem->sum16_adpd_data;
			pData.timestamp = pAdpdItem->sum_timestamp;
			if (ADPD_SUM_cb)
				ADPD_SUM_cb(&pData);
			else
				std::cout << "ADPD SUM Callback Not Initialized" << std::endl;

		}
	};
	void call(std::vector<adpd_stream_cb_data_t_4CH16> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {

			adpd_stream_cb_data_t_4CH16 *pAdpdItem = &pkt[i];
			adpd_stream_cb_data_t_4ch pData = { 0 };

			pData.seqnum = (uint16_t) sequence_num;
			pData.timestamp = (double)pAdpdItem->ch_timestamp;
			for (int i = 0; i < 4; i++)
			{
				pData.ch_data[i] =(uint32_t) pAdpdItem->ch_data[i];
			}
			if (ADPD_4CH_cb)
				ADPD_4CH_cb(&pData);
			else
				std::cout << "ADPD 4CH Callback Not Initialized" << std::endl;

		}

	};
	void call(std::vector<adpd_stream_cb_data_t_4CH32> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			adpd_stream_cb_data_t_4CH32 *pAdpdItem = &pkt[i];
			adpd_stream_cb_data_t_4ch pData = { 0 };

			pData.seqnum = sequence_num;
			pData.timestamp = pAdpdItem->ch_timestamp;

			for (int i = 0; i < 4; i++)
			{
				pData.ch_data[i] = pAdpdItem->ch_data[i];

			}
			if (ADPD_4CH_cb)
				ADPD_4CH_cb(&pData);
			else
				std::cout << "ADPD 4CH Callback Not Initialized" << std::endl;

		}

	};
};


class BatteryStreamCallback : public battery_stream_callback
{
public:
	void call(std::vector<battery_stream_cb_data_t> pkt)
	{
		battery_stream_cb_data_t *pBatInfo = &pkt[0];

		pm_sys_bat_info bat_info = { 0 };
		bat_info.bat_chrg_stat = pBatInfo->bat_chrg_stat;
		bat_info.bat_lvl = pBatInfo->bat_lvl;
		bat_info.bat_mv = pBatInfo->bat_mv;
		bat_info.bat_temp = pBatInfo->bat_temp;
		bat_info.timestamp = pBatInfo->timestamp;

		if (BATTERY_cb != NULL)
			BATTERY_cb(&bat_info);
		else
			std::cout << "Battery Callback Not Initialized" << std::endl;

	}

};

class Adpd4000StreamCallback : public adpd4000_stream_callback {
public:

	void call(std::vector<adpd4000_stream_cb_data_t> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			adpd4000_stream_cb_data_t *pAdpdItem = &pkt[i];
			adpd4000_stream_cb_data adpd4xxx_data;
			adpd4xxx_data.dark = pAdpdItem->dark;
			adpd4xxx_data.signal = pAdpdItem->signal;
			adpd4xxx_data.impulse = pAdpdItem->impulse;
			adpd4xxx_data.lit = pAdpdItem->lit;
			adpd4xxx_data.timestamp = pAdpdItem->timestamp;
			adpd4xxx_data.seq_num = sequence_num;
			adpd4xxx_data.channel_num = pAdpdItem->channel_num;
			adpd4xxx_data.adpd_d_length = pAdpdItem->adpddata_d.size();
			adpd4xxx_data.adpd_s_length = pAdpdItem->adpddata_s.size();
			adpd4xxx_data.adpd_l_length = pAdpdItem->adpddata_l.size();
			adpd4xxx_data.src = static_cast<ADPD4XXX_SOURCE>(pAdpdItem->adpd_stream);
			for (int j = 0; j < pAdpdItem->adpddata_d.size(); j++)
			{
				adpd4xxx_data.adpddata_d[j] = pAdpdItem->adpddata_d[j];

			}
			for (int k = 0; k < pAdpdItem->adpddata_s.size(); k++)
			{
				adpd4xxx_data.adpddata_s[k] = pAdpdItem->adpddata_s[k];
			}
			for (int j = 0; j < pAdpdItem->adpddata_l.size(); j++)
			{
				adpd4xxx_data.adpddata_l[j] = pAdpdItem->adpddata_l[j];

			}

			if (ADPD4XXX_cb != NULL)
				ADPD4XXX_cb(&adpd4xxx_data);
			else
				std::cout << "ADPD4xxx Callback Not Initialized" << std::endl;

		}
	}
	void call(std::vector<adpd4000_optional_stream_cb_data_t> pkt, uint16_t sequence_num)
	{
		for (uint32_t i = 0; i < pkt.size(); i++) {
			adpd4000_optional_stream_cb_data_t *pAdpdItem = &pkt[i];
			adpd4000_interrupt_stream_cb_data_t adpd4xxx_data;
			adpd4xxx_data.seq_num = sequence_num;
			adpd4xxx_data.timestamp = pAdpdItem->timestamp;
			adpd4xxx_data.data_int = pAdpdItem->data_int;
			adpd4xxx_data.level0_int = pAdpdItem->level0_int;
			adpd4xxx_data.level1_int = pAdpdItem->level1_int;
			adpd4xxx_data.tia_ch1_int = pAdpdItem->tia_ch1_int;
			adpd4xxx_data.tia_ch2_int = pAdpdItem->tia_ch2_int;

			if (ADPD4XXX_INTERRUPT_cb != NULL)
			{
				ADPD4XXX_INTERRUPT_cb(&adpd4xxx_data);
			}
			else
			{
				std::cout << "ADPD4xxx Interrupt stream Callback was Not Initialized" << std::endl;
			}


		}
	}
};

class PpgStreamCallback : public ppg_stream_callback{
public:
	
	void call(std::vector<ppg_stream_cb_data_t> pkt, uint16_t sequence_num) {
		
		for (uint32_t i = 0; i < pkt.size(); i++) {
			ppg_stream_cb_data_t *pPpgItem = &pkt[i];
			ppg_stream_cb_data pData;
			pData.adpdlibstate = pPpgItem->adpdlibstate;
			pData.confidence = pPpgItem->confidence;
			pData.hr = pPpgItem->hr;
			pData.hr_type = pPpgItem->hr_type;
			pData.rr_interval = pPpgItem->rr_interval;
			pData.timestamp = pPpgItem->timestamp;
			pData.seqnum = sequence_num;
			if (PPG_cb != NULL)
				PPG_cb(&pData);
			else
				std::cout << "PPG Callback Not Initialized" << std::endl;

		}
	};
};

class EcgStreamCallback : public ecg_stream_callback {
public:

	void call(std::vector<ecg_stream_cb_data_t> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			ecg_stream_cb_data_t *pEcgItem = &pkt[i];
			ecg_stream_cb_data pData;
			pData.datatype = pEcgItem->datatype;
			pData.ecg_data = pEcgItem->ecg_data;
			pData.HR = pEcgItem->HR;
			pData.leadsoff = pEcgItem->leadsoff;
			pData.timestamp = pEcgItem->timestamp;
			pData.seqnum = sequence_num;
			if (ECG_cb != NULL)
				ECG_cb(&pData);
			else
				std::cout << "ECG Callback Not Initialized" << std::endl;

		}
	};
};

class TemperatureStreamCallback : public temperature_stream_callback {
public:
	void call(std::vector<temperature_stream_cb_data_t> pkt, uint16_t sequence_num) {
		for (uint32_t i = 0; i < pkt.size(); i++) {
			temperature_stream_cb_data_t *pTempItem = &pkt[i];
			temperature_stream_cb_data pData;
			pData.timestamp = pTempItem->timestamp;
			pData.temp_ambient = pTempItem->temp_ambient;
			pData.temp_skin = pTempItem->temp_skin;
			if (TEMP_cb != NULL)
				TEMP_cb(&pData);
			else
				std::cout << "TEMPERATURE Callback Not Initialized" << std::endl;

		}
	};
};


class EdaStreamCallback : public eda_stream_callback {
public:

	void call(std::vector<eda_stream_cb_data_t> pkt, uint16_t sequence_num) {
		
		for (uint32_t i = 0; i < pkt.size(); i++) {
			eda_stream_cb_data_t *pEdaItem = &pkt[i];

			eda_stream_cb_data pData;
			pData.admittance_img = pEdaItem->admittance_img;
			pData.admittance_real = pEdaItem->admittance_real;
			pData.impedance_real = pEdaItem->impedance_real;
			pData.impedance_img = pEdaItem->impedance_img;
			pData.admittance_magnitude = pEdaItem->admittance_magnitude;
			pData.admittance_phase = pEdaItem->admittance_magnitude;
			pData.impedance_magnitude = pEdaItem->impedance_magnitude;
			pData.impedance_phase = pEdaItem->impedance_phase;
			pData.timestamp = pEdaItem->timestamp;
			pData.seqnum = sequence_num;
			if (EDA_cb != NULL)
				EDA_cb(&pData);
			else
				std::cout << "EDA Callback Not Initialized" << std::endl;

		}
	};
};


class SyncPPGStreamCallback : public syncppg_stream_callback {
public:
	
	void call(std::vector<syncppg_stream_cb_data_t> pkt, uint16_t sequence_num) {

		for (int i = 0; i < pkt.size(); i++) {
			syncppg_stream_cb_data_t *syncppgStreamCbData = &pkt[i];
			syncppg_stream_cb_data pData = { 0 };
			pData.ppg = syncppgStreamCbData->ppg;
			pData.x = syncppgStreamCbData->x;
			pData.y = syncppgStreamCbData->y;
			pData.z = syncppgStreamCbData->z;
			pData.timestamp = syncppgStreamCbData->timestamp;
			pData.seqnum = sequence_num;
			if (SYNCPPG_cb != NULL)
				SYNCPPG_cb(&pData);
			else
				std::cout << "SYNCPPG Callback Not Initialized" << std::endl;

		}
	};
};

static AdxlStreamCallback ADXL_Stream_cb = AdxlStreamCallback();
static AdpdStreamCallback ADPD_Stream_cb = AdpdStreamCallback();
static BatteryStreamCallback BAt_Stream_cb = BatteryStreamCallback();
static Adpd4000StreamCallback ADPD4xxx_stream_cb = Adpd4000StreamCallback();
static SyncPPGStreamCallback SYNCPPG_stream_cb = SyncPPGStreamCallback();
static PpgStreamCallback PPG_stream_cb = PpgStreamCallback();
static EcgStreamCallback ECG_stream_cb = EcgStreamCallback();
static EdaStreamCallback EDA_stream_cb = EdaStreamCallback();
static TemperatureStreamCallback TEMP_stream_cb = TemperatureStreamCallback();

static TXCallback g_phy_callback = TXCallback();
static watch mw = watch(g_phy_callback);

static add ADD_CB = NULL;

/*=========================================================================================
|									SDK TX & RX											   |
===========================================================================================*/

void  initializesdk(Tx_Callback tx_cb, sdkAlertCB sdk_cb)
{
	TX_CB = tx_cb;
	SDKALERT_CB = sdk_cb;
    printf("Initialize SDK");
	std::cout << "CPP INITIALIZE SDK" << std::endl;

}

int addcb(add cb)
{
ADD_CB = cb;
 ADD_CB(10, 20);
 return 7;
}

 void  dispatch_SDK(unsigned char *pBytes, int nLength)
{
	std::vector<uint8_t> vec_pkt(pBytes, pBytes + nLength);
#ifdef RXRAWPACKET
	std::cout << "Rx:" << std::endl;
	for (uint32_t i = 0; i < nLength; i++) {
		std::cout << " " << std::hex << std::setfill('0')
			<< std::setw(2) << (int)pBytes[i];
	}
	std::cout << std::endl;
#endif
	mw.dispatch(vec_pkt);
}


int getSDKversion()
{
    return 3;

}
void getLength(tx_length tx_cb)
{
tx_length txinst = tx_cb;

txinst();

}




/*=========================================================================================
|									PM													   |
===========================================================================================*/

 void  pm_get_system_info(pm_sys_info *pSysInfo)
{
	pm_sys_info_t pm_sysinfo = { 0 };
	mw.pm_app.get_system_info(&pm_sysinfo);
	pSysInfo->hw_id = pm_sysinfo.hw_id;
	pSysInfo->batch_id = pm_sysinfo.batch_id;
	pSysInfo->bom_id = pm_sysinfo.bom_id;
	pSysInfo->device_id = pm_sysinfo.device_id;
	pSysInfo->model_number = pm_sysinfo.model_number;
	pSysInfo->version = pm_sysinfo.version;
	pSysInfo->date = pm_sysinfo.date;

	char *macstr = (char *)malloc(sizeof(char) * sizeof(pm_sysinfo.mac_addr));
	memcpy(macstr, pm_sysinfo.mac_addr.c_str(), sizeof(pm_sysinfo.mac_addr));
	pSysInfo->mac_addr = macstr;
	delete(macstr);
}

 void  pm_get_mcu_version(uint8_t *pMCU_Type)
{
	uint8_t mcuType;
	mw.pm_app.get_mcu_version(mcuType);
	*pMCU_Type = mcuType;
}

 SDK_STATUS  pm_set_date_time(sys_date_time *pTimeInfo)
{
	ret::sdk_status status;
	sys_date_time_t timeinfo = { 0 };
	timeinfo.year = pTimeInfo->year;
	timeinfo.month = pTimeInfo->month;
	timeinfo.day = pTimeInfo->day;
	timeinfo.hour = pTimeInfo->hour;
	timeinfo.minute = pTimeInfo->minute;
	timeinfo.second = pTimeInfo->second;
	timeinfo.TZ_sec = pTimeInfo->TZ_sec;
	status = mw.pm_app.set_date_time(&timeinfo);
	return static_cast<SDK_STATUS>(status);
}

 void  pm_get_date_time(sys_date_time *pTimeInfo)
{
	sys_date_time_t timeinfo = { 0 };
	mw.pm_app.get_date_time(&timeinfo);
	pTimeInfo->year = timeinfo.year;
	pTimeInfo->month = timeinfo.month;
	pTimeInfo->day = timeinfo.day;
	pTimeInfo->hour = timeinfo.hour;
	pTimeInfo->minute = timeinfo.minute;
	pTimeInfo->second = timeinfo.second;
	pTimeInfo->TZ_sec =  timeinfo.TZ_sec;
}

 void  pm_get_battery_info(pm_sys_bat_info *pSysBatInfo)
{
	pm_sys_bat_info_t sys_battery_info = { 0 };
	mw.pm_app.get_battery_info(&sys_battery_info);
	pSysBatInfo->bat_chrg_stat = sys_battery_info.bat_chrg_stat;
	pSysBatInfo->bat_lvl = sys_battery_info.bat_lvl;
	pSysBatInfo->bat_mv = sys_battery_info.bat_mv;
	pSysBatInfo->bat_temp = sys_battery_info.bat_temp;
	pSysBatInfo->timestamp = sys_battery_info.timestamp;

}


 SDK_STATUS  pm_set_battery_threshold(uint8_t nLowLvl, uint8_t nCriticalLvl)
{
	ret::sdk_status status;
	status = mw.pm_app.set_battery_threshold(nLowLvl, nCriticalLvl);
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  pm_set_usb_power(bool enable, bool action)
{
	ret::sdk_status status;
	status = mw.pm_app.set_usb_power(enable, action);
	return static_cast<SDK_STATUS>(status);
}
 SDK_STATUS  pm_set_power_state(uint8_t nPowerState)
{
	ret::sdk_status status;
	status = mw.pm_app.set_power_state(nPowerState);
	return static_cast<SDK_STATUS>(status);
}
 SDK_STATUS  pm_ping()
{
	ret::sdk_status status;
	status = mw.pm_app.ping();
	return static_cast<SDK_STATUS>(status);
}
 SDK_STATUS  pm_enable_or_disable_battery_charger(bool enable)
{
	ret::sdk_status status;
	status = mw.pm_app.enable_or_disable_battery_charger(enable);
	return static_cast<SDK_STATUS>(status);
}
 SDK_STATUS  pm_enable_or_disable_thermistor_state_change(bool enable)
{
	ret::sdk_status status;
	status = mw.pm_app.enable_or_disable_thermistor_state_change(enable);
	return static_cast<SDK_STATUS>(status);
}

 uint8_t  pm_powerboost_m3(uint8_t nEnableState, uint8_t nOVPValue)
{
	uint8_t status;
	status = mw.pm_app.powerboost_m3(nEnableState, nOVPValue);
	return status;
}
 uint8_t  pm_powerboost_m4(uint8_t nEnableState)
{
	uint8_t status;
	status = mw.pm_app.powerboost_m4(nEnableState);
	return status;
}
 uint8_t  pm_boost_m4(uint8_t nEnableState, uint8_t  nBoostVoltage)
{
	uint8_t status;
	status = mw.pm_app.boost_m4(nEnableState, nBoostVoltage);
	return status;
}
 uint8_t  pm_read_EEPROM(eeprom_info *pEEPROM_Info)
{
	uint8_t status;
	eeprom_info_t eeprominfo = { 0 };
	status = mw.pm_app.read_EEPROM(&eeprominfo);
	pEEPROM_Info->manufacture_id = eeprominfo.manufacture_id;
	pEEPROM_Info->hw_id = eeprominfo.hw_id;
	pEEPROM_Info->bom_id = eeprominfo.bom_id;
	pEEPROM_Info->batch_id = eeprominfo.batch_id;
	pEEPROM_Info->date = eeprominfo.date;
	pEEPROM_Info->additional_detail = eeprominfo.additional_detail;
	return status;
}

 void  pm_subscribe_battery(battery_stream_cb battery_cb)
{
	BATTERY_cb = battery_cb;
	mw.pm_app.battery_stream.subscribe(BAt_Stream_cb);

}
 void  pm_unsubscribe_battery()
{
	BATTERY_cb = NULL;
	mw.pm_app.battery_stream.unsubscribe();

}


/*=========================================================================================
|									ADXL												   |
===========================================================================================*/

 void  subscribe_adxl(adxl_stream_Cb adxl_cb)
{
	ADXL_cb = adxl_cb;
	int status = mw.adxl_app.adxl_stream.subscribe(ADXL_Stream_cb);
}


/*=========================================================================================
|									ADPD												   |
===========================================================================================*/


 SDK_STATUS  adpd_clockCalibration(void)
{
	ret::sdk_status status;
	status = mw.adpd_app.calibrate_clock();
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  adpd_loadConfig(SENSOR_ADPD_DEVICE_ID device_id)
{
	ret::sdk_status status;
	status = mw.adpd_app.load_cfg(static_cast<adpd_application::SENSOR_ADPD_DEVICE_ID>(device_id));
	return static_cast<SDK_STATUS>(status);
}


 void  adpd_darkoffset(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3, uint32_t *pValue4)
{
	mw.adpd_app.darkoffset(*pValue1, *pValue2, *pValue3, *pValue4);
}

 void  adpd_floatConfig(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3)
{
	mw.adpd_app.floatconfig(*pValue1, *pValue2, *pValue3);
}

 void  adpd_getctrvalue(uint32_t *pValue1, uint32_t *pValue2, uint32_t *pValue3)
{
	mw.adpd_app.get_ctrvalue(*pValue1, *pValue2, *pValue3);
}

 void  adpd_set_slot(ADPD_SLOT_MODE_ENUM slot_a, ADPD_SLOT_MODE_ENUM slot_b)
{
	mw.adpd_app.set_slot(static_cast<adpd_application::ADPD_SLOT_MODE_ENUM_t>(slot_a), static_cast<adpd_application::ADPD_SLOT_MODE_ENUM_t>(slot_b));
}

 void  subscribe_adpd(adpd_stream_sum adpd_sum_cb, adpd_stream_4ch adpd_4ch_cb)
{
	ADPD_SUM_cb = adpd_sum_cb;
	ADPD_4CH_cb = adpd_4ch_cb;
	int status = mw.adpd_app.adpd_stream.subscribe(ADPD_Stream_cb);
}



/*=========================================================================================
|									ADPD4xxx											   |
===========================================================================================*/

 SDK_STATUS  adpd4xxx_load_cfg(void)
{
	ret::sdk_status status;
	status = mw.adpd4000_app.load_cfg();
	return static_cast<SDK_STATUS>(status);

}

 SDK_STATUS  adpd4xxx_set_slot(uint8_t  nSlotNum, uint8_t  nSlotEnableState, uint16_t  nSlotFormat, uint8_t  nChannelNum)
{
	ret::sdk_status status;
	status = mw.adpd4000_app.set_slot(nSlotNum, nSlotEnableState, nSlotFormat, nChannelNum);
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  adpd4xxx_get_slot(uint8_t nSlotNum, get_slot_info *pSlotInfo)
{
	ret::sdk_status status;
	get_slot_info_t slot_info = { 0 };
	status = mw.adpd4000_app.get_slot(nSlotNum, &slot_info);
	pSlotInfo->slot_num = slot_info.slot_num;
	pSlotInfo->channel_num = slot_info.channel_num;
	pSlotInfo->slot_enable = slot_info.slot_enable;
	pSlotInfo->slot_format = slot_info.slot_format;
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  adpd4xxx_set_slot_active(uint8_t nSlotNum, uint8_t nSlotActvState)
{
	ret::sdk_status status;
	status = mw.adpd4000_app.set_slot_active(nSlotNum, nSlotActvState);
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  adpd4xxx_get_slot_active(uint8_t nSlotNum, uint8_t *pSlotNum, uint8_t *pSlotActvState)
{
	ret::sdk_status status;
	status = mw.adpd4000_app.get_slot_active(nSlotNum, *pSlotNum, *pSlotActvState);
	return static_cast<SDK_STATUS>(status);
}

 SDK_STATUS  adpd4xxx_calibrate_clock(uint8_t nClockCalibID)
{
	ret::sdk_status status;
	status = mw.adpd4000_app.calibrate_clock(nClockCalibID);
	return static_cast<SDK_STATUS>(status);
}

 void  subscribe_adpd4xxx(ADPD4XXX_SOURCE src, adpd4xxx_stream_cb adpd4xxx_cb)
{
	ADPD4XXX_cb = adpd4xxx_cb;
	ret::sdk_status status;
	switch (src)
	{
	case ADPD4XXX_SOURCE::STREAM1:
		mw.adpd4000_app.adpd4000_stream1.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM2:
		mw.adpd4000_app.adpd4000_stream2.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM3:
		mw.adpd4000_app.adpd4000_stream3.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM4:
		mw.adpd4000_app.adpd4000_stream4.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM5:
		mw.adpd4000_app.adpd4000_stream5.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM6:
		mw.adpd4000_app.adpd4000_stream6.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM7:
		mw.adpd4000_app.adpd4000_stream7.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM8:
		mw.adpd4000_app.adpd4000_stream8.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM9:
		mw.adpd4000_app.adpd4000_stream9.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM10:
		mw.adpd4000_app.adpd4000_stream10.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM11:
		mw.adpd4000_app.adpd4000_stream11.subscribe(ADPD4xxx_stream_cb);
		break;
	case ADPD4XXX_SOURCE::STREAM12:
		mw.adpd4000_app.adpd4000_stream12.subscribe(ADPD4xxx_stream_cb);
		break;
	default:
		status = ret::sdk_status::SDK_ERR_INVALID_VALUE;
		std::cout << "Unknown ADPD4XXX source" << std::endl;
		break;
	}

}

 void  unsubscribe_adpd4xxx(ADPD4XXX_SOURCE src)
{
	ret::sdk_status status;
	switch (src)
	{
	case ADPD4XXX_SOURCE::STREAM1:
		mw.adpd4000_app.adpd4000_stream1.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM2:
		mw.adpd4000_app.adpd4000_stream2.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM3:
		mw.adpd4000_app.adpd4000_stream3.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM4:
		mw.adpd4000_app.adpd4000_stream4.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM5:
		mw.adpd4000_app.adpd4000_stream5.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM6:
		mw.adpd4000_app.adpd4000_stream6.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM7:
		mw.adpd4000_app.adpd4000_stream7.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM8:
		mw.adpd4000_app.adpd4000_stream8.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM9:
		mw.adpd4000_app.adpd4000_stream9.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM10:
		mw.adpd4000_app.adpd4000_stream10.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM11:
		mw.adpd4000_app.adpd4000_stream11.unsubscribe();
		break;
	case ADPD4XXX_SOURCE::STREAM12:
		mw.adpd4000_app.adpd4000_stream12.unsubscribe();
		break;
	default:
		status = ret::sdk_status::SDK_ERR_INVALID_VALUE;
		std::cout << "Unknown ADPD4XXX source" << std::endl;
		break;
	}

}

 void  subscribe_adpd4xxx_interrupt(adpd4xxx_interrupt_stream_cb adpd4xxx_cb)
{
	ADPD4XXX_INTERRUPT_cb = adpd4xxx_cb;
	mw.adpd4000_app.adpd4000_optional_stream.subscribe(ADPD4xxx_stream_cb);
}
 void  unsubscribe_adpd4xxx_interrupt()
{
	ADPD4XXX_INTERRUPT_cb = NULL;
	mw.adpd4000_app.adpd4000_optional_stream.unsubscribe();
}


/*=========================================================================================
|									PPG													   |
===========================================================================================*/

 SDK_STATUS  ppg_load_lcfg(SENSOR_PPG_LCFG_ID lcfg_id)
{
	ret::sdk_status status;
	status = mw.ppg_app.load_lcfg(static_cast<ppg_application::SENSOR_PPG_LCFG_ID>(lcfg_id));
	return static_cast<SDK_STATUS>(status);
}
 void  subscribe_ppg(ppg_stream_cb ppg_cb)
{
	PPG_cb = ppg_cb;
	mw.ppg_app.ppg_stream.subscribe(PPG_stream_cb);

}
 void  subscribe_syncppg(syncppg_stream_cb syncppg_cb)
{
	SYNCPPG_cb = syncppg_cb;
	mw.syncppg_app.syncppg_stream.subscribe(SYNCPPG_stream_cb);

}

/*=========================================================================================
|									EDA													   |
===========================================================================================*/
 void  subscribe_ecg(ecg_stream_cb ecg_cb)
{
	ECG_cb = ecg_cb;
	mw.ecg_app.ecg_stream.subscribe(ECG_stream_cb);
}


/*=========================================================================================
|									ECG													   |
===========================================================================================*/
 void  subscribe_eda(eda_stream_cb eda_cb)
{
	EDA_cb = eda_cb;
	mw.eda_app.eda_stream.subscribe(EDA_stream_cb);
}


 /*=========================================================================================
 |									TEMPERATURE													   |
 ===========================================================================================*/
 void subscribe_temperature(temperature_stream_cb temp_cb)
 {
	 TEMP_cb = temp_cb;
	 mw.temperature_app.temperature_stream.subscribe(TEMP_stream_cb);	 
 }


/*=========================================================================================
|							Common APPlication(ADPD/ADXL/ADPD4000/PPG)					   |
===========================================================================================*/


 void  get_version(SENSORS sensor, common_app_version *pVersion)
{
	common_app_version_t app_version = { 0 };

	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.get_version(&app_version);
		break;
	case SENSORS::ADPD:
		mw.adpd_app.get_version(&app_version);
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.get_version(&app_version);
		break;
	case SENSORS::PM:
		mw.pm_app.get_version(&app_version);
		break;
	case SENSORS::PPG:
		mw.ppg_app.get_version(&app_version);
		break;
	case SENSORS::SYNCPPG:
		mw.syncppg_app.get_version(&app_version);
		break;
	case SENSORS::ECG:
		mw.ecg_app.get_version(&app_version);
		break;
	case SENSORS::EDA:
		mw.eda_app.get_version(&app_version);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}

	pVersion->major = app_version.major;
	pVersion->minor = app_version.minor;
	pVersion->patch = app_version.patch;

	
 	int nVersize = sizeof(app_version.verstr); 
   	int nVendstrsize = sizeof(app_version.vendstr); 

  /* Stored in heap segment like other dynamically allocated things */
  	char *verstr = (char *)malloc(sizeof(char)*nVersize); 

    char *vendstr = (char *)malloc(sizeof(char)*nVendstrsize); 


	memcpy(verstr, app_version.verstr.c_str(), nVersize);
	memcpy(vendstr, app_version.vendstr.c_str(), nVendstrsize);

	pVersion->vendstr = vendstr;
	pVersion->verstr = verstr;

	delete(verstr);
	delete(vendstr);

	// memcpy(pVersion->vendstr, app_version.vendstr.c_str(), sizeof(pVersion->vendstr));
	// memcpy(pVersion->verstr, app_version.verstr.c_str(), sizeof(pVersion->verstr));
/*	strcpy_s(pVersion->vendstr, strlen(pVersion->vendstr), app_version.vendstr.c_str());
	strcpy_s(pVersion->verstr, strlen(pVersion->verstr), app_version.verstr.c_str())*/;

}

 void  read_registers(SENSORS sensor, unsigned short *pAddress, int nLength, registers *pReg)
{
	std::vector<uint16_t> reg_addr(pAddress, pAddress + nLength);
	std::vector<std::pair<uint16_t, uint16_t>> ret_vals;
	switch (sensor)
	{
	case SENSORS::ADXL:
		ret_vals = mw.adxl_app.register_read(reg_addr);
		break;
	case SENSORS::ADPD:
		ret_vals = mw.adpd_app.register_read(reg_addr);
		break;
	case SENSORS::ADPD4000:
		ret_vals = mw.adpd4000_app.register_read(reg_addr);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}

	for (int i = 0; i < ret_vals.size(); i++)
	{
		pReg->address[i] = ret_vals[i].first;
		pReg->value[i] = ret_vals[i].second;
	}

	pReg->length = nLength;
}


 void  get_dcfg(SENSORS sensor, registers *pReg)
{
	std::vector<std::pair<uint16_t, uint16_t>> ret_vals;
	switch (sensor)
	{
	case SENSORS::ADXL:
		ret_vals = mw.adxl_app.get_dcfg();
		break;
	case SENSORS::ADPD:
		ret_vals = mw.adpd_app.get_dcfg();
		break;
	case SENSORS::ADPD4000:
		ret_vals = mw.adpd4000_app.get_dcfg();
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}

	for (int i = 0; i < ret_vals.size(); i++)
	{
		pReg->address[i] = ret_vals[i].first;
		pReg->value[i] = ret_vals[i].second;
	}
	pReg->length = ret_vals.size();
}


 void  write_register(SENSORS sensor, registers *pReg)
{
	std::vector<uint16_t> reg_addr(pReg->address, pReg->address + pReg->length);
	std::vector<uint16_t> reg_val(pReg->value, pReg->value + pReg->length);

	std::vector<std::pair<uint16_t, uint16_t>> ret_vals;

	for (int i = 0; i < pReg->length; i++)
	{
		std::pair<uint16_t, uint16_t> v;
		v.first = reg_addr[i];
		v.second = reg_val[i];
		ret_vals.push_back(v);
	}

	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.register_write(ret_vals);
		break;
	case SENSORS::ADPD:
		mw.adpd_app.register_write(ret_vals);
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.register_write(ret_vals);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
}

 void   start_stream(SENSORS sensor)
{
	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.adxl_stream.start();
		break;
	case SENSORS::ADPD:
		mw.adpd_app.adpd_stream.start();
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.adpd4000_stream1.start();
		break;
	case SENSORS::PPG:
		mw.ppg_app.ppg_stream.start();
		break;
	case SENSORS::ECG:
		mw.ecg_app.ecg_stream.start();
		break;
	case SENSORS::EDA:
		mw.eda_app.eda_stream.start();
		break;
	case SENSORS::AD5940:
		mw.ad5940_app.start();
		break;
	case SENSORS::TEMPERATURE:
		mw.temperature_app.temperature_stream.start();
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
}

 void  stop_stream(SENSORS sensor)
{
	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.adxl_stream.stop();
		break;
	case SENSORS::ADPD:
		mw.adpd_app.adpd_stream.stop();
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.adpd4000_stream1.stop();
		ADPD4XXX_cb = NULL;
		break;
	case SENSORS::PPG:
		mw.ppg_app.ppg_stream.stop();
		break;
	case SENSORS::ECG:
		mw.ecg_app.ecg_stream.stop();
		break;
	case SENSORS::EDA:
		mw.eda_app.eda_stream.stop();
		break;
	case SENSORS::AD5940:
		mw.ad5940_app.stop();
		break;
	case SENSORS::TEMPERATURE:
		mw.temperature_app.temperature_stream.stop();
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
}


 void  unsubscribe_stream(SENSORS sensor)
{
	int status = 0;
	switch (sensor)
	{
	case SENSORS::ADXL:
		ADXL_cb = NULL;
		status = mw.adxl_app.adxl_stream.unsubscribe();
		break;
	case SENSORS::ADPD:
		ADPD_SUM_cb = NULL;
		ADPD_4CH_cb = NULL;
		status = mw.adpd_app.adpd_stream.unsubscribe();
		break;
	case SENSORS::PPG:
		PPG_cb = NULL;
		status = mw.ppg_app.ppg_stream.unsubscribe();
		break;
	case SENSORS::SYNCPPG:
		SYNCPPG_cb = NULL;
		status = mw.syncppg_app.syncppg_stream.unsubscribe();
		break;
	case SENSORS::PM:
		BATTERY_cb = NULL;
		status = mw.pm_app.battery_stream.unsubscribe();
		break;
	case SENSORS::ECG:
		ECG_cb = NULL;
		status = mw.ecg_app.ecg_stream.unsubscribe();
		break;
	case SENSORS::EDA:
		EDA_cb = NULL;
		status = mw.eda_app.eda_stream.unsubscribe();
		break;
	case SENSORS::TEMPERATURE:
		TEMP_cb = NULL;
		mw.temperature_app.temperature_stream.unsubscribe();
		break;
	default:
		status = ret::sdk_status::SDK_ERR_INVALID_VALUE;
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
}

 void  setTimeout(SENSORS sensor, uint16_t ntime)
{
	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.setTimeout(ntime);
		break;
	case SENSORS::ADPD:
		mw.adpd_app.setTimeout(ntime);
		break;
	case SENSORS::PPG:
		mw.ppg_app.setTimeout(ntime);
		break;
	case SENSORS::SYNCPPG:
		mw.syncppg_app.setTimeout(ntime);
		break;
	case SENSORS::PM:
		mw.pm_app.setTimeout(ntime);
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.setTimeout(ntime);
		break;
	case SENSORS::ECG:
		mw.ecg_app.setTimeout(ntime);
		break;
	case SENSORS::EDA:
		mw.eda_app.setTimeout(ntime);
		break;
	case SENSORS::TEMPERATURE:
		mw.temperature_app.setTimeout(ntime);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}

}

 uint16_t  getTimeout(SENSORS sensor)
{
	uint16_t time_intrvl = 0;
	switch (sensor)
	{
	case SENSORS::ADXL:
		time_intrvl = mw.adxl_app.GetTimeout();
		break;
	case SENSORS::ADPD:
		time_intrvl = mw.adpd_app.GetTimeout();
		break;
	case SENSORS::PPG:
		time_intrvl = mw.ppg_app.GetTimeout();
		break;
	case SENSORS::SYNCPPG:
		time_intrvl = mw.syncppg_app.GetTimeout();
		break;
	case SENSORS::PM:
		time_intrvl = mw.pm_app.GetTimeout();
		break;
	case SENSORS::ADPD4000:
		time_intrvl = mw.adpd4000_app.GetTimeout();
		break;
	case SENSORS::ECG:
		time_intrvl = mw.ecg_app.GetTimeout();
		break;
	case SENSORS::EDA:
		time_intrvl = mw.eda_app.GetTimeout();
		break;
	case SENSORS::TEMPERATURE:
		time_intrvl = mw.temperature_app.GetTimeout();
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
	return time_intrvl;
}


 void  stream_status(SENSORS sensor, uint8_t *nSubreq, uint8_t *nStartreq)
{
	uint8_t nStatus, nNumsubscribers, nStartReq;
	switch (sensor)
	{
	case SENSORS::ADXL:
		mw.adxl_app.adxl_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::ADPD:
		mw.adpd_app.adpd_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::PPG:
		mw.ppg_app.ppg_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::SYNCPPG:
		mw.syncppg_app.syncppg_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::PM:
		mw.pm_app.battery_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::ADPD4000:
		mw.adpd4000_app.adpd4000_stream1.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::ECG:
		mw.ecg_app.ecg_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::EDA:
		mw.eda_app.eda_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	case SENSORS::TEMPERATURE:
		mw.temperature_app.temperature_stream.get_status(nStatus, nNumsubscribers, nStartReq);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
	*nSubreq = nNumsubscribers;
	*nStartreq = nStartReq;
}

 void  get_algo_vendor_version(SENSORS sensor, common_app_version *pVersion)
{
	common_app_version_t app_version = { 0 };
	switch (sensor)
	{
	case SENSORS::PPG:
		mw.ppg_app.get_algo_vendor_version(&app_version);
		break;
	case SENSORS::ECG:
		mw.ecg_app.get_algo_vendor_version(&app_version);
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
	pVersion->major = app_version.major;
	pVersion->minor = app_version.minor;
	pVersion->patch = app_version.patch;

 	int nVersize = sizeof(app_version.verstr); 
   	int nVendstrsize = sizeof(app_version.vendstr); 

  /* Stored in heap segment like other dynamically allocated things */
  	char *verstr = (char *)malloc(sizeof(char)*nVersize); 

    char *vendstr = (char *)malloc(sizeof(char)*nVendstrsize); 


	memcpy(verstr, app_version.verstr.c_str(), nVersize);
	memcpy(vendstr, app_version.vendstr.c_str(), nVendstrsize);

	pVersion->vendstr = vendstr;
	pVersion->verstr = verstr;

	delete(verstr);
	delete(vendstr);
}

void getPPGAlgo(app_version *pVersion)
{
	common_app_version_t app_version = { 0 };
	mw.ppg_app.get_algo_vendor_version(&app_version);

	 int size = 20; 
  
  /* Stored in heap segment like other dynamically allocated things */
  char *str = (char *)malloc(sizeof(char)*size); 

	memcpy(str, app_version.verstr.c_str(), size);



	// memcpy(pVersion->vendstr, app_version.vendstr.c_str(), sizeof(pVersion->vendstr));
	pVersion->verstr = str;
}


 void  get_lcfg(SENSORS sensor, registers *pReg)
{
	std::vector<uint32_t> ret_vals;
	switch (sensor)
	{
	case SENSORS::PPG:
		ret_vals = mw.ppg_app.get_lcfg();
		break;
	case SENSORS::ECG:
		ret_vals = mw.ecg_app.get_lcfg();
		break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}

	for (int i = 0; i < ret_vals.size(); i++)
	{
		pReg->value[i] = ret_vals[i];
	}
	pReg->length = ret_vals.size();

}

 void  write_lcfgreg(SENSORS sensor, registers *pReg)
{
	switch (sensor)
	{
	case SENSORS::PPG:
	{
		std::vector<uint8_t> reg_addr(pReg->address, pReg->address + pReg->length);
		std::vector<uint32_t> reg_val(pReg->value, pReg->value + pReg->length);

		std::vector<std::pair<uint8_t, uint32_t>> ret_vals;

		for (int i = 0; i < pReg->length; i++)
		{
			std::pair<uint8_t, uint32_t> v;
			v.first = reg_addr[i];
			v.second = reg_val[i];
			ret_vals.push_back(v);
		}

		mw.ppg_app.lcfg_write(ret_vals);
	}
	break;
	case SENSORS::ECG:
	{
		std::vector<uint8_t> reg_addr(pReg->address, pReg->address + pReg->length);
		std::vector<uint16_t> reg_val(pReg->value, pReg->value + pReg->length);

		std::vector<std::pair<uint8_t, uint16_t>> ret_vals;

		for (int i = 0; i < pReg->length; i++)
		{
			std::pair<uint8_t, uint16_t> v;
			v.first = reg_addr[i];
			v.second = reg_val[i];
			ret_vals.push_back(v);
		}

		mw.ecg_app.lcfg_write(ret_vals);
	}
	break;
	case SENSORS::EDA:
	{
		std::vector<uint8_t> reg_addr(pReg->address, pReg->address + pReg->length);
		std::vector<uint32_t> reg_val(pReg->value, pReg->value + pReg->length);

		std::vector<std::pair<uint8_t, uint32_t>> ret_vals;

		for (int i = 0; i < pReg->length; i++)
		{
			std::pair<uint8_t, uint32_t> v;
			v.first = reg_addr[i];
			v.second = reg_val[i];
			ret_vals.push_back(v);
		}

		mw.eda_app.lcfg_write(ret_vals);
	}
	break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}


}

 void  read_lfcgreg(SENSORS sensor, unsigned short *pAddress, int nLength, registers *pReg)
{

	switch (sensor)
	{
	case SENSORS::PPG:
	{
		std::vector<uint8_t> reg_addr(pAddress, pAddress + nLength);
		std::vector<std::pair<uint8_t, uint32_t>> ret_vals;
		ret_vals = mw.ppg_app.lcfg_read(reg_addr);

		for (int i = 0; i < ret_vals.size(); i++)
		{
			pReg->address[i] = ret_vals[i].first;
			pReg->value[i] = ret_vals[i].second;
		}

		pReg->length = nLength;
	}
	break;
	case SENSORS::ECG:
	{
		std::vector<uint8_t> reg_addr(pAddress, pAddress + nLength);
		std::vector<std::pair<uint8_t, uint16_t>> ret_vals;
		ret_vals = mw.ecg_app.lcfg_read(reg_addr);

		for (int i = 0; i < ret_vals.size(); i++)
		{
			pReg->address[i] = ret_vals[i].first;
			pReg->value[i] = ret_vals[i].second;
		}

		pReg->length = nLength;
	}
	break;
	case SENSORS::EDA:
	{
		std::vector<uint8_t> reg_addr(pAddress, pAddress + nLength);
		std::vector<std::pair<uint8_t, uint32_t>> ret_vals;
		ret_vals = mw.eda_app.lcfg_read(reg_addr);

		for (int i = 0; i < ret_vals.size(); i++)
		{
			pReg->address[i] = ret_vals[i].first;
			pReg->value[i] = ret_vals[i].second;
		}

		pReg->length = nLength;
	}
	break;
	default:
		std::cout << "Unknown sensor detected" << std::endl;
		break;
	}
	
}




/*=========================================================================================
|									LEVEL 2 API											   |
===========================================================================================*/

 void  startADXL(adxl_stream_Cb adxl_cb)
{
	ADXL_cb = adxl_cb;
	mw.start_adxl(ADXL_Stream_cb);
}

 void  stopADXL()
{
	ADXL_cb = NULL;
	mw.stop_adxl();
}

 void  startADPD(adpd_stream_sum adpd_sum_cb, adpd_stream_4ch adpd_4ch_cb)
{
	ADPD_4CH_cb = adpd_4ch_cb;
	ADPD_SUM_cb = adpd_sum_cb;
	mw.start_adpd(ADPD_Stream_cb);
}

 void  stopADPD()
{
	ADPD_4CH_cb = NULL;
	ADPD_SUM_cb = NULL;
	mw.stop_adpd();
}


 void  startPPG(ppg_stream_cb ppg_cb, syncppg_stream_cb syncppg_cb)
{
	PPG_cb = ppg_cb;
	SYNCPPG_cb = syncppg_cb;
	mw.start_syncppg(PPG_stream_cb, SYNCPPG_stream_cb);
}

 void  stopPPG()
{
	PPG_cb = NULL;
	SYNCPPG_cb = NULL;
	mw.stop_syncppg();
}

 void  startADPD4xxx(adpd4xxx_stream_cb adpd4xxx_cb)
{
	ADPD4XXX_cb = adpd4xxx_cb;
	mw.start_adpd4000(ADPD4xxx_stream_cb);
}
 void  stopADPD4xxx()
{
	ADPD4XXX_cb = NULL;
	mw.stop_adpd4000();
}


 void  startECG(ecg_stream_cb ecg_cb)
{
	ECG_cb = ecg_cb;
	mw.start_ecg(ECG_stream_cb);
}
 void  stopECG()
{
	ECG_cb = NULL;
	mw.stop_ecg();
}

 void  startEDA(eda_stream_cb eda_cb)
{
	EDA_cb = eda_cb;
	mw.start_eda(EDA_stream_cb);
}
 void  stopEDA()
{
	EDA_cb = NULL;
	mw.stop_eda();
}


 void  startTEMPARATURE(temperature_stream_cb temp_cb)
 {
	 TEMP_cb = temp_cb;
	 mw.start_temperature(TEMP_stream_cb);
 }
 void  stopTEMPARATURE()
 {
	 TEMP_cb = NULL;
	 mw.stop_temperature();
 }

