import 'dart:async';

import 'dart:typed_data';

import 'package:flutter/services.dart';

import 'dart:ffi'; // For FFI
import 'dart:io';

import 'package:ffi/ffi.dart';

/*=========================================================================================
	|								Enums used for sdk C wrapper							   |
	===========================================================================================*/
enum SENSORS {
  PM,
  ADXL,
  ADPD,
  ADPD4000,
  PPG,
  SYNCPPG,
  ECG,
  EDA,
  AD5940,
  TEMPERATURE
}

enum SDK_STATUS {
  SDK_OK,
  SDK_ERR,
  SDK_ERR_INVALID,
  SDK_DEVICE_ERR,
  SDK_PACKET_TIMEOUT,
}

/*=========================================================================================
	|								Structures used for sdk C wrapper						   |
	===========================================================================================*/

class adxl_stream_t extends Struct {
  @Uint16()
  int sequence_number;

  @Double()
  double timestamp;

  @Int16()
  int x;

  @Int16()
  int y;

  @Int16()
  int z;


  factory adxl_stream_t.allocate(int sequence_number,
  double timestamp,
  int x,
  int y,
  int z) =>
      allocate<adxl_stream_t>().ref
        ..sequence_number = sequence_number
        ..timestamp = timestamp
        ..x = x
        ..y = y
        ..z = z;
}



class sys_date_time extends Struct {
  @Uint16()
  int year;

  @Uint8()
  int month;

  @Uint8()
  int day;

  @Uint8()
  int hour;

  @Uint8()
  int minute;

  @Uint8()
  int second;

  @Uint32()
  int TZ_sec;


  factory sys_date_time.allocate(int year,
      int month,
      int day,
      int hour,
      int minute,
      int second,
      int TZ_sec
  ) =>
      allocate<sys_date_time>().ref
        ..year = year
        ..month = month
        ..day = day
        ..hour = hour
        ..minute = minute
        ..second = second
        ..TZ_sec = TZ_sec;
}



class pm_sys_info extends Struct {
  @Uint16()
  int version;

  Pointer<Utf8> mac_addr;

  @Uint32()
  int device_id;

  @Uint32()
  int model_number;

  @Uint16()
  int hw_id;

  @Uint16()
  int bom_id;

  @Uint8()
  int batch_id;

  @Uint32()
  int date;


  factory pm_sys_info.allocate(int version,
      Pointer<Utf8> mac_addr,
      int device_id,
      int model_number,
      int hw_id,
      int bom_id,
      int batch_id,
      int date
      ) =>
      allocate<pm_sys_info>().ref
        ..version = version
        ..mac_addr = mac_addr
        ..device_id = device_id
        ..model_number = model_number
        ..hw_id = hw_id
        ..bom_id = bom_id
        ..batch_id = batch_id
  ..date = date;
}


class pm_sys_bat_info extends Struct {
  @Double()
  double timestamp;

  @Uint8()
  int bat_chrg_stat;

  @Uint8()
  int bat_lvl;

  @Uint16()
  int bat_mv;

  @Uint16()
  int bat_temp;


  factory pm_sys_bat_info.allocate(double timestamp,
      int bat_chrg_stat,
      int bat_lvl,
      int bat_mv,
      int bat_temp
  ) =>
      allocate<pm_sys_bat_info>().ref
        ..timestamp = timestamp
        ..bat_chrg_stat = bat_chrg_stat
        ..bat_lvl = bat_lvl
        ..bat_mv = bat_mv
        ..bat_temp = bat_temp;
}



class eeprom_info extends Struct {
  @Uint32()
  int manufacture_id;

  @Uint16()
  int hw_id;

  @Uint16()
  int bom_id;

  @Uint8()
  int batch_id;

  @Uint32()
  int date;

  @Uint32()
  int additional_detail;

  factory eeprom_info.allocate(int manufacture_id, int hw_id, int bom_id,
          int batch_id, int date, int additional_detail) =>
      allocate<eeprom_info>().ref
        ..manufacture_id = manufacture_id
        ..hw_id = hw_id
        ..bom_id = bom_id
        ..batch_id = batch_id
        ..date = date
        ..additional_detail = additional_detail;
}




class get_slot_info extends Struct {
  @Uint8()
  int slot_num;

  @Uint8()
  int slot_enable;

  @Uint16()
  int slot_format;

  @Uint8()
  int channel_num;

  factory get_slot_info.allocate(int  slot_num,
      int  slot_enable,
      int slot_format,
      int  channel_num) =>
      allocate<get_slot_info>().ref
        ..slot_num = slot_num
        ..slot_enable = slot_enable
        ..slot_format = slot_format
        ..channel_num = channel_num;
}




class adpd4000_interrupt_stream_cb_data_t extends Struct {
  @Uint16()
  int seq_num;

  @Double()
  double timestamp;

  @Uint16()
  int data_int;

  @Uint16()
  int level0_int;

  @Uint16()
  int level1_int;

  @Uint16()
  int tia_ch1_int;

  @Uint16()
  int tia_ch2_int;


  factory adpd4000_interrupt_stream_cb_data_t.allocate(int seq_num,
  double  timestamp,
      int  data_int,
      int  level0_int,
      int  level1_int,
      int  tia_ch1_int,
      int  tia_ch2_int
      ) =>
      allocate<adpd4000_interrupt_stream_cb_data_t>().ref
        ..seq_num = seq_num
        ..timestamp = timestamp
        ..data_int = data_int
        ..level0_int = level0_int
        ..level1_int = level1_int
        ..tia_ch1_int = tia_ch1_int
        ..tia_ch2_int = tia_ch2_int;
}




class ppg_stream_cb_data extends Struct {
  @Uint16()
  int seqnum;

  @Double()
  double timestamp;

  @Uint16()
  int adpdlibstate;

  @Float()
  double hr;

  @Float()
  double confidence;

  @Uint16()
  int hr_type;

  @Uint16()
  int rr_interval;


  factory ppg_stream_cb_data.allocate(int seqnum,
  double timestamp,
      int adpdlibstate,
  double hr,
      double confidence,
      int hr_type,
      int rr_interval

  ) =>
      allocate<ppg_stream_cb_data>().ref
        ..seqnum = seqnum
        ..timestamp = timestamp
        ..adpdlibstate = adpdlibstate
        ..hr = hr
        ..confidence = confidence
        ..hr_type = hr_type
        ..rr_interval = rr_interval;
}


class syncppg_stream_cb_data extends Struct {
  @Uint16()
  int seqnum;

  @Double()
  double timestamp;

  @Uint32()
  int ppg;

  @Int16()
  int x;

  @Int16()
  int y;

  @Int16()
  int z;



  factory syncppg_stream_cb_data.allocate(int seqnum,
  double timestamp,
  int ppg,
  int x,
  int y,
  int z
      ) =>
      allocate<syncppg_stream_cb_data>().ref
        ..seqnum = seqnum
        ..timestamp = timestamp
        ..ppg = ppg
        ..x = x
        ..y = y
        ..z = z;
}



class adpd_stream_cb_data_t_sum extends Struct {
  @Uint16()
  int seqnum;

  @Double()
  double timestamp;

  @Uint32()
  int adpd_data;


  factory adpd_stream_cb_data_t_sum.allocate(int seqnum,
  double    timestamp,
  int  adpd_data
  ) =>
      allocate<adpd_stream_cb_data_t_sum>().ref
        ..seqnum = seqnum
        ..timestamp = timestamp
        ..adpd_data = adpd_data;
}





class eda_stream_cb_data extends Struct {

  @Uint16()
  int seqnum;

  @Double()
  double timestamp;

  @Double()
  double admittance_real;

  @Double()
  double admittance_img;


  @Double()
  double impedance_real;


  @Double()
  double impedance_img;

  @Double()
  double admittance_magnitude;


  @Double()
  double admittance_phase;


  @Double()
  double impedance_magnitude;


  @Double()
  double impedance_phase;




  factory eda_stream_cb_data.allocate(int seqnum,
  double timestamp,
  double admittance_real,
  double admittance_img,
  double impedance_real,
  double impedance_img,
  double admittance_magnitude,
  double admittance_phase,
  double impedance_magnitude,
  double impedance_phase
      ) =>
      allocate<eda_stream_cb_data>().ref
        ..seqnum = seqnum
        ..timestamp = timestamp
        ..admittance_real = admittance_real
        ..admittance_img = admittance_img
        ..impedance_real = impedance_real
        ..impedance_img = impedance_img
        ..admittance_magnitude = admittance_magnitude
        ..admittance_phase = admittance_phase
        ..impedance_magnitude = impedance_magnitude
        ..impedance_phase = impedance_phase;
}


class ecg_stream_cb_data extends Struct {
  @Uint16()
  int seqnum;

  @Double()
  double timestamp;

  @Uint8()
  int datatype;

  @Uint8()
  int leadsoff;

  @Uint8()
  int HR;

  @Uint16()
  int ecg_data;



  factory ecg_stream_cb_data.allocate(int seqnum,
  double timestamp,
      int datatype,
  int leadsoff,
      int HR,
      int ecg_data
  ) =>
      allocate<ecg_stream_cb_data>().ref
        ..seqnum = seqnum
        ..timestamp = timestamp
        ..datatype = datatype
        ..leadsoff = leadsoff
        ..HR = HR
        ..ecg_data = ecg_data;
}




class temperature_stream_cb_data extends Struct {

  @Double()
  double timestamp;

  @Float()
  double temp_skin;

  @Float()
  double temp_ambient;



  factory temperature_stream_cb_data.allocate(double timestamp,
      double temp_skin,
      double temp_ambient
  ) =>
      allocate<temperature_stream_cb_data>().ref
        ..timestamp = timestamp
        ..temp_skin = temp_skin
        ..temp_ambient = temp_ambient;
}




class common_app_version extends Struct {
  @Uint16()
  int major;

  @Uint16()
  int minor;

  @Uint16()
  int patch;

  // @size(20)
  Pointer<Utf8> verstr;

  // @InlineArray(80)
  Pointer<Utf8> vendstr;

  factory common_app_version.allocate(int major, int minor, int patch,
          Pointer<Utf8> verstr, Pointer<Utf8> vendstr) =>
      allocate<common_app_version>().ref
        ..major = major
        ..minor = minor
        ..patch = patch
        ..verstr = verstr
        ..vendstr = vendstr;
}

class app_version extends Struct {
//  @Uint16()
//  int major;
//
//  @Uint16()
//  int minor;
//
//  @Uint16()
//  int patch;

  Pointer<Utf8> verstr;

  factory app_version.allocate(Pointer<Utf8> verstr) =>
      allocate<app_version>().ref..verstr = verstr;
}



/*=========================================================================================
	|							CLASSES			   |
	===========================================================================================*/

final DynamicLibrary nativeGen3SDK = Platform.isAndroid
    ? DynamicLibrary.open("libfl_gen3sdk.so")
    : DynamicLibrary.process();

typedef tx_length = Void Function();

typedef subtract_func = Void Function(Pointer a);
typedef Subtract = void Function(Pointer a);

typedef AddCbNative = Int32 Function(Pointer cb);
typedef AddCb = int Function(Pointer cb);

typedef Add = Void Function(Int32 a, Int32 b);

typedef tx_callback = Void Function(Pointer<Uint8> pBytes, Int32 nLength);

typedef sdkAlert_callback = Void Function(Int32 nLength);

typedef TX_FUNC = Void Function(Pointer a, Pointer b);
typedef tx_Func = void Function(Pointer a, Pointer b);

typedef EMPTY_func = Void Function();
typedef empty = void Function();

typedef DISPATCH_FUNC = Void Function(Pointer<Uint8> a, Int32 nLength);
typedef dispatch_Func = void Function(Pointer<Uint8> a, int nLength);

typedef EEPROM_INFO_FUNC = Uint8 Function(Pointer<eeprom_info> pEEPROM_Info);
typedef eeprom_info_func = int Function(Pointer<eeprom_info> pEEPROM_Info);

typedef APP_VERSION_FUNC = Void Function(
    Uint8 sts, Pointer<common_app_version> pApp_Info);
typedef app_version_func = void Function(
    int sts, Pointer<common_app_version> pApp_Info);

final initializeSDK =
    nativeGen3SDK.lookupFunction<TX_FUNC, tx_Func>("initializesdk");

class FlGen3sdk {
  static const MethodChannel _channel = const MethodChannel('fl_gen3sdk');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

//  int _id = 0;
//  final _i2cb = <int, Callback>{};
//  final _cb2i = <Callback, int>{};

  static void _tx_length() {
    print("Callback occured");
  }

  static void add(int a, int b) {
    print('$a $b');
    return;
  }

  static void _initiaLizeSDK(Pointer<Uint8> pBytes, int nLength) {
//    final Pointer<Uint8> frameData = allocate<Uint8>(count: nLength); // Allocate a pointer large enough.
    Uint8List pointerList = pBytes.asTypedList(nLength);

    print("List $pointerList");
  }

  static void _sdkAlertMessage(int alertmsg) {
    print("ALERT MESSAGE -- $alertmsg");
  }

  void callAddMethod() {
    initializeSDK(Pointer.fromFunction<tx_callback>(_initiaLizeSDK),
        Pointer.fromFunction<sdkAlert_callback>(_sdkAlertMessage));

    List<int> rcvdData = [
      195,
      0,
      199,
      5,
      0,
      66,
      0,
      0,
      20,
      0,
      1,
      0,
      2,
      0,
      0,
      0,
      45,
      72,
      67,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      65,
      68,
      73,
      32,
      72,
      82,
      77,
      59,
      0,
      0,
      0,
      0,
      32,
      79,
      0,
      32,
      88,
      68,
      1,
      32,
      0,
      0,
      0,
      0,
      1,
      0,
      0,
      0,
      67,
      49,
      1,
      8,
      176,
      118,
      1,
      32,
      0,
      0,
      0,
      0
    ];

//    final pointer = Pointer.allo

    // Uint8List list = Uint8List.fromList(rcvdData);

//    final srcBuffer = UIn.toPointer(data);

    // Pointer<Utf8> RCVD_data = Utf8.toUtf8(rcvdData.toString());

    Pointer<Uint8> rcvdDataptr = allocate<Uint8>(count: rcvdData.length);

//    RCVD_data.asTypedList(rcvdData.length);
//
    for (int i = 0; i < rcvdData.length; i++) {
      rcvdDataptr[i] = rcvdData[i];
      // print(rcvdDataptr[i]);
    }

//    RCVD_data.value = list;

//    print(rcvdData.toString());
//    RCVD_data.cast();

//    RCVD_data.value = rcvdData;
    var dispatchSDK = nativeGen3SDK
        .lookupFunction<DISPATCH_FUNC, dispatch_Func>("dispatch_SDK");

    dispatchSDK(rcvdDataptr, rcvdData.length);

    free(rcvdDataptr);

    var app_info_call =
        nativeGen3SDK.lookupFunction<APP_VERSION_FUNC, app_version_func>(
            "get_algo_vendor_version");

    Pointer<common_app_version> appVersionstruct = allocate();

    // appVersionstruct.ref.verstr = allocate<Utf8>(count: 20);

    app_info_call(4, appVersionstruct);

    // print("APP Version INFO  ${Utf8.fromUtf8(appVersionstruct.ref.verstr)} ");

    //  ${Utf8.fromUtf8(appVersionstruct.ref.vendstr)}

    print(
        "APP Version INFO ${appVersionstruct.ref.major} ${appVersionstruct.ref.minor} ${appVersionstruct.ref.patch} ${Utf8.fromUtf8(appVersionstruct.ref.vendstr)} ${Utf8.fromUtf8(appVersionstruct.ref.verstr)}");

    free(appVersionstruct);

    var eeprom_info_call = nativeGen3SDK
        .lookupFunction<EEPROM_INFO_FUNC, eeprom_info_func>("pm_read_EEPROM");

    Pointer<eeprom_info> eeprom_struct = allocate();

    eeprom_info_call(eeprom_struct);

    print(
        "EEPROM INFO ${eeprom_struct.ref.hw_id} ${eeprom_struct.ref.bom_id} ${eeprom_struct.ref.batch_id} ${eeprom_struct.ref.manufacture_id}");

    free(eeprom_struct);

    final pm_bat = nativeGen3SDK.lookupFunction<EMPTY_func, empty>("stopPPG");

    pm_bat();

//    var library = DynamicLibrary.open("tets-dll.dll");
    var addcb = nativeGen3SDK.lookupFunction<AddCbNative, AddCb>("addcb");
//    Pointer<NativeFunction<Add>> addptr = addcb;

    print(addcb(Pointer.fromFunction<Add>(add)));

    var tx_len =
        nativeGen3SDK.lookupFunction<subtract_func, Subtract>("getLength");

    tx_len(Pointer.fromFunction<tx_length>(_tx_length));

//    var stop_adxl = nativeGen3SDK.lookupFunction<EMPTY_func, empty>("stopPPG");
//
//    stop_adxl();
//    print(addcb(fromFunction<Add>(add)));
  }
}
